/*
 * Next Framework http://www.nextframework.org
 * Copyright (C) 2009 the original author or authors.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * You may obtain a copy of the license at
 * 
 *     http://www.gnu.org/copyleft/lesser.html
 * 
 */
/*
 * Criado em 22/03/2005
 *
 */
package org.nextframework.authorization.impl;

import java.security.Principal;

import org.nextframework.authorization.AuthorizationDAO;
import org.nextframework.authorization.Role;
import org.nextframework.authorization.User;
import org.nextframework.authorization.UserLocator;
import org.nextframework.core.standard.RequestContext;
import org.nextframework.core.web.DefaultWebRequestContext;
import org.nextframework.core.web.WebRequestContext;
import org.nextframework.exception.NextException;
import org.nextframework.view.menu.MenuTag;


/**
 * 
 * Implementacao da interface para localizar os Users
 * 
 * @author rogelgarcia
 */
public class UserLocatorImpl implements UserLocator{
    
	protected AuthorizationDAO authorizationDAO;

    public AuthorizationDAO getAuthorizationDAO() {
		return authorizationDAO;
	}

	public void setAuthorizationDAO(AuthorizationDAO authorizationDAO) {
		this.authorizationDAO = authorizationDAO;
	}

	public User getUser(RequestContext request) {
        User user = request.getUser();
        if(user == null){
            Principal userPrincipal = ((WebRequestContext)request).getUserPrincipal();
            if (userPrincipal!=null) {
                String username = userPrincipal.getName();
                if (username != null) {
                    user = authorizationDAO.findUserByLogin(username);
                    //limpar cache dos menus
                    if(request instanceof WebRequestContext){
                    	((WebRequestContext)request).getSession().setAttribute(MenuTag.MENU_CACHE_MAP, null);
                    }
                    if (user == null) {    
                        throw new NextException(
                                "Usurio no encontrado no banco de dados: "
                                        + username);
                    }
                } else {
                    // O CDIGO PROVAVELMENTE NO CHEGAR AQUI
                    // userPrincipal.getName() no deve retornar null
                    throw new RuntimeException("Erro inesperado: Algoritmo invlido em UserLocatorImpl");
                }
            } else {
                //NINGUEM LOGADO
                user = null;
            }
            //TODO TROCAR PARA DEFAULTREQUESTCONTEXT QUANDO TIVER
            ((DefaultWebRequestContext)request).setUser(user);
        }
        
        return user;
    }

	public void loadPermissions(WebRequestContext requestContext) {
		if (requestContext.getSession().getAttribute("NEXT_LOADPERMISSIONS")==null) {
			User user = getUser(requestContext);
			if (user != null) {
				Role[] findUserRoles = authorizationDAO.findUserRoles(user);
				for (Role role : findUserRoles) {
					requestContext.getSession().setAttribute("ROLE_" + role.getName().toUpperCase(), true);
				}
				requestContext.getSession().setAttribute("NEXT_LOADPERMISSIONS", true);
			}
		}
	}
}
