/*
 * Next Framework http://www.nextframework.org
 * Copyright (C) 2009 the original author or authors.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * You may obtain a copy of the license at
 * 
 *     http://www.gnu.org/copyleft/lesser.html
 * 
 */
package org.nextframework.classmanager;

import java.io.File;
import java.io.IOException;

import javax.servlet.ServletContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nextframework.exception.NextException;


/**
 * Registra as classes quando estiver em um ambiente J2EE.
 * Fornece um ClassManager
 * @author rogelgarcia
 *
 */
public class WebClassRegister {

	private static final String CLASS_MANAGER_ATTRIBUTE = ClassManager.class.getName();
	
	private static Log log = LogFactory.getLog(WebClassRegister.class);
	
	private WebClassRegister(){
		
	}
	
	public static ClassManager getClassManager(ServletContext context, String ignorePackage) throws IOException{
		ClassManager classManager = (ClassManager) context.getAttribute(CLASS_MANAGER_ATTRIBUTE);
		if(classManager == null){
			long x = System.currentTimeMillis();
			classManager = new ClassManagerImpl();
			//TODO TROCAR O MDO COMO ADQUIRE AS CLASSES EM MODO WEB
			String path = context.getRealPath(File.separator+"WEB-INF"+File.separator+"classes");
			if(path == null){ //Bug 0008
				throw new NextException("Could not find directory /WEB-INF/classes");
			}
			String[] classesInPackage = ClassLocator.getClassesNameInPackage(path, File.separator);
			
			
			for (String name : classesInPackage) {
				try {
					//ignorar as classes do pacote a ser ignorado e as classes com test no nome
					if(!name.startsWith(ignorePackage)){
						classManager.registerClass(Class.forName(name));	
					}
					
				} catch (ClassNotFoundException e) {
					// se a classe nao for encontrada, nao fazer nada
				}
			}				
			context.setAttribute(CLASS_MANAGER_ATTRIBUTE, classManager);
			x = System.currentTimeMillis() - x;
			log.debug("Classes registradas em "+x+" milisegundos");
		}
		return classManager;
	}
}
