/*
 * Next Framework http://www.nextframework.org
 * Copyright (C) 2009 the original author or authors.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * You may obtain a copy of the license at
 * 
 *     http://www.gnu.org/copyleft/lesser.html
 * 
 */
package org.nextframework.controller;

import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.nextframework.authorization.AuthorizationHandlerInterceptor;
import org.nextframework.classmanager.ClassManager;
import org.nextframework.classmanager.WebClassRegister;
import org.nextframework.core.web.init.AnnotationsXmlWebApplicationContext;
import org.springframework.beans.BeansException;
import org.springframework.context.ApplicationContext;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.HandlerMapping;
import org.springframework.web.servlet.ViewResolver;

/**
 * @author rogelgarcia
 * @since 22/01/2006
 * @version 1.1
 */
public class DispatcherServlet extends org.springframework.web.servlet.DispatcherServlet {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	
	public DispatcherServlet(){
		
	}

	@Override
	protected void doService(HttpServletRequest request, HttpServletResponse response) throws Exception {
		request.setAttribute("CONFIG", getServletConfig());
		super.doService(request, response);
	}

	@SuppressWarnings("unchecked")
	@Override
	protected List getDefaultStrategies(ApplicationContext context, Class strategyInterface) throws BeansException {
		List defaultStrategies = super.getDefaultStrategies(context, strategyInterface);
		if (HandlerMapping.class.isAssignableFrom(strategyInterface)) {
			AnnotationsHandlerMapping annotationsHandlerMapping = new AnnotationsHandlerMapping();
			annotationsHandlerMapping.setInterceptors(getInterceptors());
			annotationsHandlerMapping.setModule("/" + getServletName());
			annotationsHandlerMapping.setApplicationContext(context);
			defaultStrategies.add(annotationsHandlerMapping);
		}
		if(ViewResolver.class.isAssignableFrom(strategyInterface)){
			CompositeViewResolver compositeViewResolver = new CompositeViewResolver();
			compositeViewResolver.setApplicationContext(context);
			compositeViewResolver.setParameterName("bodyPage");
			compositeViewResolver.setPrefix("/WEB-INF/jsp/"+getServletName()+"/");
			compositeViewResolver.setSuffix(".jsp");
			compositeViewResolver.setBaseView("/WEB-INF/jsp/"+getServletName()+"/base.jsp");
			defaultStrategies.add(0, compositeViewResolver);
		}
		return defaultStrategies;
	}

	@SuppressWarnings("unchecked")
	protected HandlerInterceptor[] getInterceptors() {
		List<HandlerInterceptor> interceptorsList = new ArrayList<HandlerInterceptor>();
		try {
			ClassManager classManager = WebClassRegister.getClassManager(getServletContext(), "org.nextframework");
			List<Class<?>> interceptorsClasses = new ArrayList<Class<?>>( Arrays.asList(classManager.getAllClassesOfType(HandlerInterceptor.class)));
			for (Iterator iterator = interceptorsClasses.iterator(); iterator.hasNext();) {
				Class<?> class1 = (Class<?>) iterator.next();
				if((class1.getModifiers() & Modifier.ABSTRACT) == 0){
					//nao  abstrato
					interceptorsList.add((HandlerInterceptor) class1.newInstance());
				}
			}
			interceptorsList.add(new AuthorizationHandlerInterceptor());
		} catch (Exception e) {
			//se ocorrer alguma exceo apenas logar o erro
			e.printStackTrace();
		}
		HandlerInterceptor[] interceptors = interceptorsList.toArray(new HandlerInterceptor[interceptorsList.size()]);
		return interceptors;
	}

	@Override
	public Class<?> getContextClass() {
		return AnnotationsXmlWebApplicationContext.class;
	}

}
