/*
 * Next Framework http://www.nextframework.org
 * Copyright (C) 2009 the original author or authors.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * You may obtain a copy of the license at
 * 
 *     http://www.gnu.org/copyleft/lesser.html
 * 
 */
package org.nextframework.core.standard.init;

import java.io.IOException;
import java.lang.annotation.Annotation;
import java.util.Arrays;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nextframework.bean.AnnotatedBeanRegister;
import org.nextframework.bean.TypeBeanRegister;
import org.nextframework.classmanager.ClassManager;
import org.nextframework.classmanager.ClassRegister;
import org.nextframework.core.standard.Next;
import org.springframework.beans.BeansException;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.context.support.FileSystemXmlApplicationContext;


/**
 * ApplicationContext que registra beans no spring atravs de annotations
 * (No confunda com o ApplicationContext do pacote 
 * org.nextframework.core.ApplicationContext)
 * @author rogelgarcia
 * @since 21/01/2006
 * @version 1.1
 */
public class AnnotationsXmlApplicationContext extends FileSystemXmlApplicationContext {

	protected static final Log log = LogFactory.getLog(AnnotationsXmlApplicationContext.class);
	
	protected AnnotatedBeanRegister[] beanRegisters = new AnnotatedBeanRegister[0];

	private TypeBeanRegister[] typeBeanRegisters = new TypeBeanRegister[0];
	
	/**
	 * Configura esse application context com os registradores de bean
	 * @param beanRegisters
	 */
	public void setBeanRegisters(AnnotatedBeanRegister[] beanRegisters) {
		this.beanRegisters = beanRegisters;
	}



	@Override
	protected void loadBeanDefinitions(DefaultListableBeanFactory beanFactory) throws IOException {
		
		// carregar primeiro as classe anotadas...
		// as classes no XML podero sobrescrever as anotadas
		ClassManager classManager = ClassRegister.getClassManager();
		
		for (AnnotatedBeanRegister register : beanRegisters) {
			Class<? extends Annotation> annotationClass = register.getAnnotationClass();
			Class<?>[] classesWithAnnotation = classManager.getClassesWithAnnotation(annotationClass);
			register.registerBeans(Arrays.asList(classesWithAnnotation), beanFactory);
		}
		for (TypeBeanRegister register : typeBeanRegisters) {
			Class<?> beanClass = register.getBeanClass();
			Class<?>[] allClassesOfType = classManager.getAllClassesOfType(beanClass);
			register.registerBeans(Arrays.asList(allClassesOfType), beanFactory);
		}
		if (Next.isInApplicationContext()) {
			Next.getApplicationContext().getConfig().setDefaultListableBeanFactory(beanFactory);
		}
		super.loadBeanDefinitions(beanFactory);
	}



	public AnnotationsXmlApplicationContext(String[] configLocations) throws BeansException {
		super(configLocations, false);
	}



	public void setTypeBeanRegisters(TypeBeanRegister[] typeBeanRegisters) {
		this.typeBeanRegisters = typeBeanRegisters;
		
	}
}
