/*
 * Next Framework http://www.nextframework.org
 * Copyright (C) 2009 the original author or authors.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * You may obtain a copy of the license at
 * 
 *     http://www.gnu.org/copyleft/lesser.html
 * 
 */
package org.nextframework.util;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.EntityMode;
import org.hibernate.collection.AbstractPersistentCollection;
import org.hibernate.collection.PersistentBag;
import org.hibernate.collection.PersistentCollection;
import org.hibernate.engine.SessionImplementor;
import org.hibernate.proxy.HibernateProxy;
import org.hibernate.proxy.LazyInitializer;
import org.nextframework.core.standard.Next;
import org.nextframework.persistence.QueryBuilder;
import org.springframework.orm.hibernate3.HibernateTemplate;

public class HibernateUtils {
	
	Log log = LogFactory.getLog(this.getClass());

	public Object getId(HibernateTemplate hibernateTemplate, Object bean){
		Class<? extends Object> class1 = bean.getClass();
		while(class1.getName().contains("$$")){
			class1 = class1.getSuperclass();
		}
		return hibernateTemplate.getSessionFactory().getClassMetadata(class1).getIdentifier(bean, EntityMode.POJO);
	}

	public boolean isLazy(Object value) {
		if(value instanceof HibernateProxy){
			LazyInitializer hibernateLazyInitializer = ((HibernateProxy)value).getHibernateLazyInitializer();
			return hibernateLazyInitializer.isUninitialized();
		} else if(value instanceof PersistentCollection){
			return !((PersistentCollection)value).wasInitialized();
		}
		return false;
	}

	public <E> E getLazyValue(E value) {
		if(value instanceof HibernateProxy){
			LazyInitializer hibernateLazyInitializer = ((HibernateProxy)value).getHibernateLazyInitializer();
			if(hibernateLazyInitializer.isUninitialized()){
				try {
					Class<?> superclass = value.getClass().getSuperclass();
					Serializable identifier = hibernateLazyInitializer.getIdentifier();

					if (identifier != null) {
						Object newValue = new QueryBuilder(Next.getObject(HibernateTemplate.class)).from(superclass).idEq(identifier).unique();
						if(newValue != null){
							value = (E) newValue;
						} else {
							log.warn("No foi possvel encontrar "+superclass.getSimpleName()+" com identificador "+identifier+" no banco de dados");										
						}
						log.warn("Carregando objeto da classe " + superclass.getName() + " sob demanda. Utilize leftOuterJoinFetch para evitar esse problema");
					}
				} catch (IllegalArgumentException e) {
				} catch (SecurityException e) {
				}
			}
		} else if(value instanceof PersistentCollection){
			try {
				PersistentCollection collection = (PersistentCollection) value;
				if (!collection.wasInitialized()) {
					Object owner = collection.getOwner();
					String role = collection.getRole();
					value = (E) Util.beanFacotries.getDAOForClass(owner.getClass()).loadCollection(owner, role.substring(role.lastIndexOf('.') + 1));
					System.out.println("COLECAO LAZY " + owner.getClass().getSimpleName() + "." + role);
				}
			} catch (Exception e) {
				//se nao conseguir carregar o valor lazy, no fazer nada
			}
		}
		return value;
	}
}
