/*
 * Next Framework http://www.nextframework.org
 * Copyright (C) 2009 the original author or authors.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * 
 * You may obtain a copy of the license at
 * 
 *     http://www.gnu.org/copyleft/lesser.html
 * 
 */
package org.nextframework.view;

import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import org.hibernate.SessionFactory;
import org.nextframework.controller.resource.Resource;
import org.nextframework.core.standard.Next;
import org.nextframework.persistence.FileDAO;
import org.nextframework.types.File;
import org.springframework.beans.factory.support.DefaultListableBeanFactory;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.orm.hibernate3.HibernateTemplate;
import org.springframework.transaction.support.TransactionTemplate;


public class DownloadFileServlet extends HttpServlet {

	private static final long serialVersionUID = 1L;

	public static final String DOWNLOAD_FILE_PATH = "/DOWNLOADFILE";
	
	public static final String DOWNLOAD_FILE_MAP = "NEXT_DOWNLOAD_FILE_MAP";
	
	private static long tempFileId = -1;
	
	public synchronized static long getNewTempFileId(){
		return tempFileId--;
	}

	public void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {

		Long cdfile;
		try {
			cdfile = extractCdfile(request);
		}
		catch (Exception e) {
        	response.sendError(HttpServletResponse.SC_NOT_FOUND);
        	return;
		}
		
		// Checa se h permisso
		if (DownloadFileServlet.getCdfile(request.getSession(), cdfile)==null) {
			response.sendError(HttpServletResponse.SC_FORBIDDEN);
			return;
		}
		
		// Obtm o contedo
		Resource resource = getResource(request, cdfile);
        if(resource == null){
        	response.sendError(HttpServletResponse.SC_NOT_FOUND);
        	return;
        }

        response.setContentType(resource.getContentType());
		response.setHeader("Content-Disposition","attachment; filename=\""+ resource.getFileName() + "\";");
		//response.setHeader("Last-Modified", );
		if (resource.getSize()>=0) {
			response.setContentLength(resource.getSize());
		}

		response.getOutputStream().write(resource.getContents());
		response.flushBuffer();
	}

	@Override
	protected long getLastModified(HttpServletRequest request) {
		try {
			return getLastModified(request, extractCdfile(request));
		}
		catch (Exception e) {
			return 0;
		}
	}
	
	protected Long extractCdfile(HttpServletRequest request) throws Exception {
		String requestURI = request.getRequestURI();
		Pattern pattern = Pattern.compile(".+?/(-?[0-9]+)");
		Matcher matcher = pattern.matcher(requestURI);
		if (matcher.find()) {
			return new Long(matcher.group(1));
		}
		else {
			throw new Exception("URL invlida");
		}
	}

	public static void addCdfile(HttpSession session, Long cdfile) {
		getMap(session).put(cdfile, cdfile);
	}

	public static Long getCdfile(HttpSession session, Long cdfile) {
		return getMap(session).get(cdfile);
	}

	private static HashMap<Long, Long> getMap(HttpSession session) {
		@SuppressWarnings("unchecked")
		HashMap<Long, Long> map = (HashMap<Long, Long>) session.getAttribute(DOWNLOAD_FILE_MAP);
		
		if (map == null) {
			map = new HashMap<Long, Long>();
			session.setAttribute(DOWNLOAD_FILE_MAP, map);
		}

		return map;
	}

	protected Resource getResource(HttpServletRequest request, Long cdfile){
		DefaultListableBeanFactory defaultListableBeanFactory = Next.getApplicationContext().getConfig().getDefaultListableBeanFactory();
		Map beansOfType;
		do{
			beansOfType = defaultListableBeanFactory.getBeansOfType(FileDAO.class); 
			if(beansOfType.size() == 0){
				defaultListableBeanFactory = (DefaultListableBeanFactory)defaultListableBeanFactory.getParentBeanFactory();
			} else{
				break;
			}
		} while(defaultListableBeanFactory != null);
		
		if(beansOfType.size() == 0){
			//se nao tem FileDAO.. criar um default se s tiver um tipo de arquivo
			final Class<?>[] classes = Next.getApplicationContext().getClassManager().getAllClassesOfType(File.class);
			if(classes.length == 1){
				//TODO REFATORAR ESSE CDIGO, ELE SE REPETE NO GENERICDAO
				FileDAO fileDAO = new FileDAO(classes[0], true);
				fileDAO.setHibernateTemplate(Next.getObject(HibernateTemplate.class));
				fileDAO.setJdbcTemplate(Next.getObject(JdbcTemplate.class));
				fileDAO.setSessionFactory(Next.getObject(SessionFactory.class));
				fileDAO.setTransactionTemplate(Next.getObject(TransactionTemplate.class));
				beansOfType.put("fileDAO", fileDAO);//satisfazemos um FileDAO default
			}
		}
		
		if(cdfile < 0){
			File file = load(cdfile);
			return returnFile(file);
		} else if(beansOfType.size() == 1){
			FileDAO<?> fileDAO = (FileDAO<?>) beansOfType.values().iterator().next();
			Class<?>[] allClassesOfTypeFile = Next.getApplicationContext().getClassManager().getAllClassesOfType(File.class);
			if(allClassesOfTypeFile.length == 1){
				File file;
				try {
					file = (File)allClassesOfTypeFile[0].newInstance();
					file.setCdfile(cdfile);
					file = fileDAO.loadWithContents(file);
					return returnFile(file);
				} catch (InstantiationException e) {
				} catch (IllegalAccessException e) {
				}
			}
		}
		throw new RuntimeException("Estenda a classe DownloadFileServlet e sobrescreva o mtodo getResource, ou crie um DAO que extenda FileDAO");
	}

	private Resource returnFile(File file) {
		Resource resource = new Resource(file.getContenttype(), file.getName(), file.getContent());
		Long size = file.getSize();
		if (size != null) {
			resource.setSize(size.intValue());
		}
		return resource;
	}
	
	protected long getLastModified(HttpServletRequest request, Long cdfile){
		return -1;
	}

	public static File load(long tempFileId){
		java.io.File tempFile = new java.io.File(System.getProperty("java.io.tmpdir"), Next.getApplicationName()+"_tempFileObject"+tempFileId+".next");
		try {
			ObjectInputStream in = new ObjectInputStream(new FileInputStream(tempFile));
			Object obj = in.readObject();
			in.close();
			return (File) obj;
		} catch (FileNotFoundException e) {
			throw new RuntimeException(e);
		} catch (IOException e) {
			throw new RuntimeException(e);
		} catch (ClassNotFoundException e) {
			throw new RuntimeException(e);
		}
	}
	
	public static void persist(File value, long tempFileId) throws IOException {
		//TODO UNIFICAR O LOCAL DE SALVAR DE LER OS ARQUIVOS TEMPORARIOS
		java.io.File tempFile = new java.io.File(System.getProperty("java.io.tmpdir"), Next.getApplicationName()+"_tempFileObject"+tempFileId+".next");
		System.out.println("TEMPORARY FILE    "+tempFile.getAbsolutePath());
		ObjectOutputStream out = new ObjectOutputStream(new FileOutputStream(tempFile));
		out.writeObject(value);
		out.flush();
		out.close();
	}

}
